use strict;

sub write_file {
  my ($file, $out) = @_;

  my $fh;
  open $fh, '>', $file or die;
  binmode $fh;
  print $fh $out;
  close $fh;
}

sub init_atr {
  my ($imageSize, $sectorSize) = (0x16800, 0x80);
  my $atr = pack "vvvv", 0x296, $imageSize >> 4, $sectorSize, $imageSize >> 20;
  $atr .= "\x00" x 8;
  $atr .= "\x00" x $imageSize;
  $atr;
}

sub read_lines {
  my ($file) = @_;
  my @lines = ();
  open my $fh, $file or die;
  while (<$fh>) {
    chomp;
    push @lines, $_;
  }
  @lines;
}

sub read_figs {
  my ($file) = @_;

  my @lines = read_lines($file);

  my ($atr, $lastScr, $disk) = (init_atr(), 0, 0);
  for (my $line = 1; $line < scalar(@lines); $line++) {
    $_ = $lines[$line - 1];
    if (/^SCR #(\d+) $/) {
      my $scr = $1;
      if ($scr == 3) {
        write_file("disk${disk}.atr", $atr);
        $disk++;
        $atr = init_atr();
      }

      print STDERR "$line: SCR: $_\n" if !($scr == 3 || $scr == $lastScr + 1);
      $lastScr = $scr;

      #print STDERR "$_\n";
      $line++;
      $_ = $lines[$line - 1];
      print STDERR "$line: CRLF1: $_\n" if $_ ne '';
      my $screen = "\x20" x 0x400;
      for (my $i = 0; $i < 16; $i++) {
        $line++;
        $_ = $lines[$line - 1];
        s/NEXT ([ @!])/ZEXT \1/g;
        my $num = sprintf "%2d ", $i;
        my $data = substr($_, 3);
        print STDERR "$line: LEN: $_\n" if length($_) > 64 + 3;
        if (substr($_, 0, 3) ne $num) {
          print STDERR "$line: NUMB: $_\n";
          $i = 16;
        }

        substr($screen, $i * 64, length($data)) = $data;
      }

      substr($atr, $scr * 0x400 + 0x10, 0x400) = $screen;
      $line++;
      $_ = $lines[$line - 1];
      print STDERR "$line: CRLF2: $_\n" if $_ ne '';
    }
  }

  write_file('disk2.atr', $atr);
}

read_figs(@ARGV);
